<?php

/**
 * @file
 * provides admin pages for Comment Easy Reply module.
 *
 */

/**
 * The module's settings form.
 */
function comment_easy_reply_settings_form($form, &$form_state) {
  $form['settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Comment Easy Reply settings'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  $form['settings'] += _comment_easy_reply_settings_form($form_state);
  $form['#validate'][] = 'comment_easy_reply_settings_form_validate';
  $active_types = _comment_easy_reply_get_active_node_types();
  if (empty($active_types)) {
    drupal_set_message(t('Comment Easy Reply is currently not enabled for any node type. Control node type status on !status_page.',
    array('!status_page' => l(t('status page'), 'admin/config/content/comment-easy-reply/status'))), 'warning');
  }
  return system_settings_form($form);
}

/**
 * Generates the settings form.
 *
 * @param array $form_state
 *   The form values.
 * @param string $node_type
 *   The node type. It is used in node settings form, while in general settings
 *   page, the value is NULL.
 */
function _comment_easy_reply_settings_form(&$form_state, $node_type = NULL) {
  $form = array();
  $suffix = '';
  if (!is_null($node_type) && !empty($node_type)) {
    $suffix = '_' . $node_type;
  }
  $form['behaviours'] = array(
    '#type' => 'fieldset',
    '#title' => '',
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  $form['tooltip'] = array(
    '#type' => 'fieldset',
    '#title' => '',
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  $form['fragment'] = array(
    '#type' => 'fieldset',
    '#title' => '',
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  $form['tokens'] = array(
    '#type' => 'fieldset',
    '#title' => '',
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  $form['behaviours']['comment_easy_reply_reply_from_numlink' . $suffix] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow users to reply to a comment by clicking on comment #NUM link.'),
    '#description' => t('If this option is not selected, the comment #NUM link will be a permalink.'),
    '#default_value' => variable_get('comment_easy_reply_reply_from_numlink' . $suffix, COMMENT_EASY_REPLY_DEFAULT_REPLY_FROM_NUMLINK),
  );
  $modes = array(
    'number' => t('Display link as comment number (Eg: #45)'),
    'username' => t('Display link as username (Eg: @username)'),
  );
  $form['behaviours']['comment_easy_reply_link_mode' . $suffix] = array(
    '#type' => 'radios',
    '#title' => t('Link mode'),
    '#options' => $modes,
    '#description' => t('Select the mode the reference to a comment will be shown into comment body'),
    '#default_value' => variable_get('comment_easy_reply_link_mode' . $suffix, COMMENT_EASY_REPLY_DEFAULT_LINK_MODE),
  );
  $form['tooltip']['comment_easy_reply_replytip_activated' . $suffix] = array(
    '#type' => 'checkbox',
    '#title' => t('Activate reply tooltip.'),
    '#description' => t('This checkbox controls the tooltip that appears passing the mouse over comment #NUM link.'),
    '#default_value' => variable_get('comment_easy_reply_replytip_activated' . $suffix, COMMENT_EASY_REPLY_DEFAULT_REPLYTIP_ACTIVATED),
  );
  $form['tooltip']['comment_easy_reply_replytip_text' . $suffix] = array(
    '#type' => 'textfield',
    '#title' => t('Text inside comment reply tooltip'),
    '#description' => t('This text will be used as link inside any comment reply tooltip.'),
    '#default_value' => variable_get('comment_easy_reply_replytip_text' . $suffix, COMMENT_EASY_REPLY_DEFAULT_REPLYTIP_TEXT),
  );
  $form['tooltip']['comment_easy_reply_referrertip_text' . $suffix] = array(
    '#type' => 'textarea',
    '#title' => t('Text inside comment referrer tooltip'),
    '#description' => t('This text will be used to compose referrer comment
    text inside any comment tooltip.'), '#default_value' =>
    variable_get('comment_easy_reply_referrertip_text' . $suffix, COMMENT_EASY_REPLY_DEFAULT_REFERRERTIP_TEXT),
  );
  $form['fragment']['comment_easy_reply_fragment_activated' . $suffix] = array(
    '#type' => 'checkbox',
    '#title' => t('Activate the custom fragment.'),
    '#description' => t("This checkbox allows to change the default fragment in comment's permalink, title and links from #comment-CID to #reply-NUM."),
    '#default_value' => variable_get('comment_easy_reply_fragment_activated' . $suffix, COMMENT_EASY_REPLY_DEFAULT_FRAGMENT_ACTIVATED),
  );
  $form['fragment']['comment_easy_reply_fragment_text' . $suffix] = array(
    '#type' => 'textfield',
    '#title' => t('Text of the fragment'),
    '#description' => t('This text will be used as link inside any comment reply tooltip.'),
    '#default_value' => variable_get('comment_easy_reply_fragment_text' . $suffix, COMMENT_EASY_REPLY_DEFAULT_FRAGMENT_TEXT),
  );
  if (module_exists('token')) {
    $form['tokens']['tokens'] = array(
      '#theme' => 'token_tree',
      '#token_types' => array('comment', 'comment_easy_reply'),
    );
  }
  else {
    $form['tokens']['tokens'] = array(
      '#markup' => '<p>' . t('Enable the <a href="@drupal-token">Token module</a> to view all the available tokens.',
      array(
        '@drupal-token' => 'http://drupal.org/project/token')
      ) . '</p>',
    );
  }
  $filters = array('' => 'None');
  $filter_info = filter_formats();
  foreach ($filter_info as $name => $filter) {
    $filters[$name] = $filter->name;
  }
  if (!empty($filters)) {
    $description = t('Apply a filter to comment text contained inside tooltips.');
    $form['tooltip']['comment_easy_reply_filter' . $suffix] = array(
      '#title' => 'Tooltip filter',
      '#description' => $description,
      '#type' => 'select',
      '#options' => $filters,
      '#default_value' => variable_get('comment_easy_reply_filter' . $suffix, ''),
    );
  }
  return $form;
}

/**
 * Validate callback function for module's settings form.
 */
function comment_easy_reply_settings_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  $fragment_active = $values['comment_easy_reply_fragment_activated'];
  if ($fragment_active) {
    $text = $values['comment_easy_reply_fragment_text'];
    if (empty($text)) {
      form_set_error('comment_easy_reply_fragment_text', t('Fragment text cannot be empty if activated.'));
    }
    else {
      $error = FALSE;
      $messages = _comment_easy_reply_validate_fragment_text($text, $error);
      if ($error) {
        foreach ($messages as $msg) {
          form_set_error('comment_easy_reply_fragment_text', $msg);
        }
      }
    }
  }
}

/**
 * Page callback for Comment Easy Reply status report.
 *
 * @return string
 *   The HTML of the page.
 */
function comment_easy_reply_status_report_page() {
  $variables = array();
  $node_types = node_type_get_names();
  if (isset($_GET['enable'])) {
    $type_enable = check_plain($_GET['enable']);
    if ($type_enable && in_array($type_enable, array_keys($node_types))) {
      $actives = _comment_easy_reply_get_active_node_types();
      if (!in_array($type_enable, $actives)) {
        $actives[] = $type_enable;
        _comment_easy_reply_set_active_node_types($actives);
      }
      drupal_set_message(t('Comment Easy Reply has been enabled for node type %type',
      array('%type' => $node_types[$type_enable])));
    }
  }
  if (isset($_GET['disable'])) {
    $type_disable = check_plain($_GET['disable']);
    if ($type_disable && in_array($type_disable, array_keys($node_types))) {
      $actives = _comment_easy_reply_get_active_node_types();
      foreach ($actives as $pos => $type) {
        if ($type == $type_disable) {
          unset($actives[$pos]);
        }
      }
      _comment_easy_reply_set_active_node_types($actives);
      drupal_set_message(t('Comment Easy Reply has been disabled for node type %type',
      array('%type' => $node_types[$type_disable])));
    }
  }
  $variables['comment_easy_reply_node_types'] = variable_get('comment_easy_reply_node_types', array());
  return theme('comment_easy_reply_status_report_page', $variables);
}

/**
 * Returns HTML for the module's status report.
 *
 * @param array $variables
 *   An associative array containing:
 *   - comment_easy_reply_node_types: An array of node types with Comment Easy
 *     Reply enabled on.
 *
 * @ingroup themeable
 */
function theme_comment_easy_reply_status_report_page($variables) {
  $rows = array();
  $headers = array(
    'Node type',
    'Current status',
    'Default settings overrided',
    'Settings details',
    'Actions');
  $node_types = node_type_get_names();
  foreach ($node_types as $node_type => $label) {
    $override = '';
    $links = array();
    if (in_array($node_type, $variables['comment_easy_reply_node_types'])) {
      $title = t('OK');
      $class = 'ok';
      $active = '<strong>' . t('enabled')  . '</strong>';
      if (_comment_easy_reply_is_node_type_override_active($node_type)) {
        $override = t('Node type specific settings used.');
      }
      else {
        $override = t('Default settings used.');
        $override .= '<br />(' . l(t('Override default settings'), 'admin/structure/types/manage/' . $node_type) . ')';
      }
      $links[] = l(t('Disable'), 'admin/config/content/comment-easy-reply/status', array('query' => array('disable' => $node_type)));
      $links[] = l(t('Edit'), 'admin/structure/types/manage/' . $node_type);
    }
    else {
      $title = t('Warning');
      $class = 'warning';
      $active = t('disabled');
      $links[] = l(t('Enable'), 'admin/config/content/comment-easy-reply/status', array('query' => array('enable' => $node_type)));
    }
    $row = array(
      array(
        'data' => $label,
      ),
      array(
        'data' => $active,
      ),
      array(
        'data' => $override,
      ),
      array(
        'data' => theme('comment_easy_reply_node_type_settings_report', array('node_type' => $node_type)),
      ),
      array(
        'data' => theme('item_list', array('items' => $links)),
      ),
    );
    $rows[] = array('data' => $row, 'class' => array($class));
  }
  return theme('table', array(
    'rows' => $rows,
    'sticky' => FALSE,
    'attributes' => array('class' => array('system-status-report')),
    'header' => $headers,
  ));
}

/**
 * Returns HTML for a node type specific settings configuration.
 *
 * @param array $variables
 *   An associative array containing:
 *   - node_type: The node type machine name.
 */
function theme_comment_easy_reply_node_type_settings_report($variables) {
  $settings = array();
  if (!_comment_easy_reply_is_node_type_active($variables['node_type'])) {
    return '';
  }
  $names = array(
    'comment_easy_reply_reply_from_numlink',
    'comment_easy_reply_replytip_activated',
    'comment_easy_reply_fragment_activated',
    'comment_easy_reply_link_mode',
    'comment_easy_reply_filter',
    'comment_easy_reply_replytip_text',
    'comment_easy_reply_referrertip_text',
    'comment_easy_reply_fragment_text',
  );
  foreach ($names as $name) {
    $value = _comment_easy_reply_get_settings_value($name, $variables['node_type']);
    $label = '';
    switch ($name) {
      case 'comment_easy_reply_reply_from_numlink':
        if ($value) {
          $value = t('enabled');
        }
        else {
          $value = t('disabled');
        }
        $label = t('Allow users to reply to a comment by clicking on comment #NUM link:');
        break;

      case 'comment_easy_reply_replytip_activated':
        if ($value) {
          $value = t('enabled');
        }
        else {
          $value = t('disabled');
        }
        $label = t('Reply tooltip on comment number permalink:');
        break;

      case 'comment_easy_reply_fragment_activated':
        if ($value) {
          $value = t('enabled');
        }
        else {
          $value = t('disabled');
        }
        $label = t('Custom url fragment:');
        break;

      case 'comment_easy_reply_link_mode':
        $label = t('Referrer link type:');
        break;

      case 'comment_easy_reply_replytip_text':
        $label = t('Text inside comment reply tooltip:');
        break;

      case 'comment_easy_reply_referrertip_text':
        $label = t('Text inside comment referrer tooltip:');
        break;

      case 'comment_easy_reply_fragment_text':
        $label = t('Url fragment custom text:');
        break;

      case 'comment_easy_reply_filter':
        $label = t('Tooltip filter:');
        $filter_info = filter_formats();
        foreach ($filter_info as $name => $filter) {
          if ($name == $value) {
            $value = $filter->name;
            break;
          }
        }
        break;

    }
    $item = $label . ' <strong>' . $value . '</strong>';
    $settings[] = array(
      'data' => $item,
    );
  }
  return theme('item_list', array('items' => $settings));
}
